Class: RASN2::ValueNotation::Error
- Inherits:
-
Error
- Object
- Error
- RASN2::ValueNotation::Error
- Defined in:
- lib/rasn2/value_notation.rb
Overview
Error raised when parsing or emitting value notation fails
Error raised when parsing or emitting value notation fails